package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.overtime.BpmOAOvertimeCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.overtime.BpmOAOvertimeRespVO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOAOvertimeConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAOvertimeDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOAOvertimeService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 加班申请 Controller
 *
 * @author 符溶馨

 */
@Tag(name = "管理后台 - OA 加班申请")
@RestController
@RequestMapping("/bpm/oa/overtime")
@Validated
public class BpmOAOvertimeController {

    @Resource
    private BpmOAOvertimeService overtimeService;

    @PostMapping("/create")
    @Operation(summary = "创建请求申请")
    public CommonResult<Long> createOvertime(@Valid @RequestBody BpmOAOvertimeCreateReqVO createReqVO) {

        return success(overtimeService.createOvertime(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得加班申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOAOvertimeRespVO> getOvertime(@RequestParam("id") Long id) {

        BpmOAOvertimeDO overtime = overtimeService.getOvertime(id);

        return success(BpmOAOvertimeConvert.INSTANCE.convert(overtime));
    }
}
