package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.regular.BpmOARegularCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.regular.BpmOARegularRespVO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOARegularConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOARegularDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOARegularService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 转正申请 Controller
 *
 * @author 符溶馨

 */
@Tag(name = "管理后台 - OA 转正申请")
@RestController
@RequestMapping("/bpm/oa/regular")
@Validated
public class BpmOARegularController {

    @Resource
    private BpmOARegularService regularService;

    @PostMapping("/create")
    @Operation(summary = "创建请求申请")
    public CommonResult<Long> createRegular(@Valid @RequestBody BpmOARegularCreateReqVO createReqVO) {

        return success(regularService.createRegular(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得转正申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOARegularRespVO> getRegular(@RequestParam("id") Long id) {

        BpmOARegularDO regular = regularService.getRegular(id);

        return success(BpmOARegularConvert.INSTANCE.convert(regular));
    }
}
