package cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.contract;

import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;

import java.time.LocalDate;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY;

/**
 * 合同审批 创建 Request VO
 *
 * @author 符溶馨
 */
@Schema(description = "管理后台 - 合同审批创建 Request VO")
public class BpmOAContractCreateReqVO {

    @Schema(description = "合同名称", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "合同名称不能为空")
    private String contractName;

    @Schema(description = "合同编号", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "合同编号不能为空")
    private String contractNumber;

    @Schema(description = "签约日期", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "签约日期不能为空")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY)
    private LocalDate signingDate;

    @Schema(description = "我方公司名称", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    private String mCompanyName;

    @Schema(description = "我方负责人", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    private String mHeadName;

    @Schema(description = "对方公司名称", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    private String oCompanyName;

    @Schema(description = "对方负责人", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    private String oHeadName;

    @Schema(description = "上传文件", requiredMode = Schema.RequiredMode.REQUIRED)
    private List<UploadUserFile> fileItems;

    public BpmOAContractCreateReqVO() {
    }

    public BpmOAContractCreateReqVO(String contractName, String contractNumber, LocalDate signingDate, String mCompanyName, String mHeadName, String oCompanyName, String oHeadName, List<UploadUserFile> fileItems) {
        this.contractName = contractName;
        this.contractNumber = contractNumber;
        this.signingDate = signingDate;
        this.mCompanyName = mCompanyName;
        this.mHeadName = mHeadName;
        this.oCompanyName = oCompanyName;
        this.oHeadName = oHeadName;
        this.fileItems = fileItems;
    }

    public String getContractName() {
        return contractName;
    }

    public void setContractName(String contractName) {
        this.contractName = contractName;
    }

    public String getContractNumber() {
        return contractNumber;
    }

    public void setContractNumber(String contractNumber) {
        this.contractNumber = contractNumber;
    }

    public LocalDate getSigningDate() {
        return signingDate;
    }

    public void setSigningDate(LocalDate signingDate) {
        this.signingDate = signingDate;
    }

    public String getmCompanyName() {
        return mCompanyName;
    }

    public void setmCompanyName(String mCompanyName) {
        this.mCompanyName = mCompanyName;
    }

    public String getmHeadName() {
        return mHeadName;
    }

    public void setmHeadName(String mHeadName) {
        this.mHeadName = mHeadName;
    }

    public String getoCompanyName() {
        return oCompanyName;
    }

    public void setoCompanyName(String oCompanyName) {
        this.oCompanyName = oCompanyName;
    }

    public String getoHeadName() {
        return oHeadName;
    }

    public void setoHeadName(String oHeadName) {
        this.oHeadName = oHeadName;
    }

    public List<UploadUserFile> getFileItems() {
        return fileItems;
    }

    public void setFileItems(List<UploadUserFile> fileItems) {
        this.fileItems = fileItems;
    }
}
