package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

/**
 * OA 备用金申请 DO
 *
 *
 * @author 符溶馨

 */
@TableName(value ="bpm_oa_imprest", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAImprestDO extends BaseDO {

    /**
     * 是否已报销 0否 1是 2进行中
     */
    public static final Integer FLAG_FALSE = 0;
    public static final Integer FLAG_TRUE = 1;
    public static final Integer IN_PROGRESS = 2;

    /**
     * 出差表单主键
     */
    @TableId
    private Long id;

    /**
     * 申请人的用户编号
     *
     * 关联 AdminUserDO 的 id 属性
     */
    private Long userId;

    /**
     * 费用类型
     */
    private String type;

    /**
     * 申请事由
     */
    private String reason;

    /**
     * 申请金额
     */
    private BigDecimal amount;

    /**
     * 使用日期
     */
    private LocalDate date;

    /**
     * 报销流程业务编号
     */
    private Long reimbursementId;

    /**
     * 申请的结果
     *
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 报销与否状态
     * 0：未报销
     * 1：已报销
     * 2：进行中
     */
    private Integer status;

    /**
     * 对应的流程编号
     *
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems ;
}
