package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.procure.ProcureDetail;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import com.baomidou.mybatisplus.annotation.KeySequence;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import java.time.LocalDateTime;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.TIME_ZONE_DEFAULT;

/**
 * OA 采购申请 DO
 *
 * @author 姚君
 */
@TableName(value ="bpm_oa_procure", autoResultMap = true)
@KeySequence("bpm_oa_procure_seq") // 用于 Oracle、PostgreSQL、Kingbase、DB2、H2 数据库的主键自增。如果是 MySQL 等数据库，可不写。
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAProcureDO extends BaseDO {
    /**
     * 是否已支付 0否 1是 2进行中
     */
    public static final Integer FLAG_FALSE = 0;
    public static final Integer FLAG_TRUE = 1;
    public static final Integer IN_PROGRESS = 2;


    /**
     * 报销表单主键
     */
    @TableId
    private Long id;
    /**
     * 申请人的用户编号
     */
    private Long userId;
    /**
     * 申请事由
     */
    private String reason;
    /**
     * 采购类型 （根据字典的来）
     */
    private Integer procureType;
    /**
     * 期望交付日期
     */
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND, timezone = TIME_ZONE_DEFAULT)
    private LocalDateTime expectedDeliveryDate;
    /**
     * 采购明细数据JSON
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<ProcureDetail> procureDetailJson;
    /**
     * 备注
     */
    private String remarks;
    /**
     * 采购总金额
     */
    private String totalMoney;
    /**
     * 采购总金额大写
     */
    private String totalMoneyChinese;

    /**
     * 是否已支付 0否 1是 2进行中
     */
    private Integer payFlag;

    /**
     * 附件信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems;
    /**
     * 采购结果
     */
    private Integer result;
    /**
     * 流程实例的编号
     */
    private String processInstanceId;

}
