package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.cash.BpmOACashCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOACashConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOACashDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOACashMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_CASH_NOT_EXISTS;

/**
 * OA 现金支出 Service 实现类
 *
 * @author 符溶馨

 */
@Service
@Validated
public class BpmOACashServiceImpl extends BpmOABaseService implements BpmOACashService{

    /**
     * OA 现金支出对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_cash";

    @Resource
    private BpmOACashMapper cashMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Override
    public Long createCash(Long userId, BpmOACashCreateReqVO createReqVO) {

        //插入OA 出差申请
        BpmOACashDO cash = BpmOACashConvert.INSTANCE.convert(createReqVO).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        cashMapper.insert(cash) ;

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(cash.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 请假单中
        cashMapper.updateById(new BpmOACashDO().setId(cash.getId()).setProcessInstanceId(processInstanceId));

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return cash.getId();
    }

    @Override
    public void updateCashResult(Long id, Integer result) {

        validateLeaveExists(id);
        cashMapper.updateById(new BpmOACashDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (cashMapper.selectById(id) == null) {
            throw exception(OA_CASH_NOT_EXISTS);
        }
    }

    @Override
    public BpmOACashDO getCash(Long id) {

        return cashMapper.selectById(id);
    }
}
