package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.evection.BpmOAEvectionCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.upload.UploadUserFile;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOAEvectionConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAEvectionDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAEvectionMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_EVECTION_NOT_EXISTS;

/**
 * OA 出差申请 Service 实现类
 *
 * @author 符溶馨

 */
@Service
@Validated
public class BpmOAEvectionServiceImpl extends BpmOABaseService implements BpmOAEvectionService {

    /**
     * OA 出差对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_evection";

    @Resource
    private BpmOAEvectionMapper evectionMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Override
    public Long createEvection(Long userId, BpmOAEvectionCreateReqVO createReqVO) {

        //插入OA 出差申请
        BpmOAEvectionDO evection = BpmOAEvectionConvert.INSTANCE.convert(createReqVO).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        evectionMapper.insert(evection) ;

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(evection.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 请假单中
        evectionMapper.updateById(new BpmOAEvectionDO().setId(evection.getId()).setProcessInstanceId(processInstanceId));

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return evection.getId();
    }

    @Override
    public void updateEvectionResult(Long id, Integer result) {

        validateLeaveExists(id);
        evectionMapper.updateById(new BpmOAEvectionDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (evectionMapper.selectById(id) == null) {
            throw exception(OA_EVECTION_NOT_EXISTS);
        }
    }

    @Override
    public BpmOAEvectionDO getEvection(Long id) {

        return evectionMapper.selectById(id);
    }
}
