package cn.iocoder.yudao.module.infra.service.file;

import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.BpmFileUploadReqVO;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.FilePageReqVO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.BpmFileDO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import org.springframework.web.multipart.MultipartFile;

/**
 * 文件 Service 接口
 *

 */
public interface FileService {

    /**
     * 获得文件分页
     *
     * @param pageReqVO 分页查询
     * @return 文件分页
     */
    PageResult<FileDO> getFilePage(FilePageReqVO pageReqVO);

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param name 文件名称
     * @param path 文件路径
     * @param content 文件内容
     * @return 文件路径
     */
    String createFile(String name, String path, byte[] content);

    /**
     * 保存文件，并返回文件对象
     *
     * @param file 上传文件对象
     * @return 文件路径
     */
    BpmFileDO createReturnFile(MultipartFile file);

    /**
     * 删除文件
     *
     * @param id 编号
     */
    void deleteFile(Long id) throws Exception;

    /**
     * 获得文件内容
     *
     * @param configId 配置编号
     * @param path 文件路径
     * @return 文件内容
     */
    byte[] getFileContent(Long configId, String path) throws Exception;


    /**
     * 删除文件
     *
     * @param url 文件URL地址
     */
    void deleteFile(String url) throws Exception;

    /**
     * 删除bpm文件
     * @param url
     * @throws Exception
     */
    void deleteBpmFile(String url) throws Exception;

    /**
     * 更新文件的流程实例ID
     * @param reqVO
     * @throws Exception
     */
    void uploadBpmFileProcessInstanceId(BpmFileUploadReqVO reqVO) throws  Exception ;
}
