package cn.iocoder.yudao.module.system.controller.admin.worklog;

import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstancePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceSaveReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogInstanceDO;
import cn.iocoder.yudao.module.system.service.worklog.LogInstanceService;

@Tag(name = "管理后台 - 日志实例的拓展")
@RestController
@RequestMapping("/system/log-instance")
@Validated
public class LogInstanceController {

    @Resource
    private LogInstanceService logInstanceService;

    @PostMapping("/create")
    @Operation(summary = "创建日志实例的拓展")
    @PreAuthorize("@ss.hasPermission('system:log-instance:create')")
    public CommonResult<Long> createLogInstance(@Valid @RequestBody LogInstanceSaveReqVO createReqVO) {
        return success(logInstanceService.createLogInstance(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新日志实例的拓展")
    @PreAuthorize("@ss.hasPermission('system:log-instance:update')")
    public CommonResult<Boolean> updateLogInstance(@Valid @RequestBody LogInstanceSaveReqVO updateReqVO) {
        logInstanceService.updateLogInstance(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除日志实例的拓展")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('system:log-instance:delete')")
    public CommonResult<Boolean> deleteLogInstance(@RequestParam("id") Long id) {
        logInstanceService.deleteLogInstance(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得日志实例的拓展")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:log-instance:query')")
    public CommonResult<LogInstanceRespVO> getLogInstance(@RequestParam("id") Long id) {
        LogInstanceDO logInstance = logInstanceService.getLogInstance(id);
        return success(BeanUtils.toBean(logInstance, LogInstanceRespVO.class));
    }

    @GetMapping("/my-page")
    @Operation(summary = "获得我的日志分页列表", description = "在【我的日志】菜单中，进行调用")
    @PreAuthorize("@ss.hasPermission('system:log-instance:query')")
    public CommonResult<PageResult<LogInstanceRespVO>> getMyLogInstancePage(@Valid LogInstancePageReqVO pageReqVO) {

        return success(logInstanceService.getMyProcessInstancePage(getLoginUserId(), pageReqVO));
    }

    @GetMapping("/page")
    @Operation(summary = "获得日志实例的拓展分页")
    @PreAuthorize("@ss.hasPermission('system:view-log:query')")
    public CommonResult<PageResult<LogInstanceRespVO>> getLogInstancePage(@Valid LogInstancePageReqVO pageReqVO) {
        PageResult<LogInstanceDO> pageResult = logInstanceService.getLogInstancePage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, LogInstanceRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出日志实例的拓展 Excel")
    @PreAuthorize("@ss.hasPermission('system:log-instance:export')")
    @OperateLog(type = EXPORT)
    public void exportLogInstanceExcel(@Valid LogInstancePageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<LogInstanceDO> list = logInstanceService.getLogInstancePage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "日志实例的拓展.xls", "数据", LogInstanceRespVO.class,
                        BeanUtils.toBean(list, LogInstanceRespVO.class));
    }
}