package cn.iocoder.yudao.module.system.service.comment;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.app.comment.dto.CommentDTO;
import cn.iocoder.yudao.module.system.controller.app.comment.vo.CommentPageListVO;
import cn.iocoder.yudao.module.system.controller.app.comment.vo.WorkLogCommentPageReqVO;
import cn.iocoder.yudao.module.system.controller.app.comment.vo.WorkLogCommentSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.comment.WorkLogCommentDO;
import cn.iocoder.yudao.module.system.dal.mysql.comment.WorkLogCommentMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 工作日志评论 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class WorkLogCommentServiceImpl implements WorkLogCommentService {

    @Resource
    private WorkLogCommentMapper workLogCommentMapper;

    @Override
    public Long createWorkLogComment(WorkLogCommentSaveReqVO createReqVO) {
        // 插入
        WorkLogCommentDO workLogComment = BeanUtils.toBean(createReqVO, WorkLogCommentDO.class);
        workLogCommentMapper.insert(workLogComment);
        // 返回
        return workLogComment.getId();
    }

    @Override
    public void updateWorkLogComment(WorkLogCommentSaveReqVO updateReqVO) {
        // 校验存在
//        validateWorkLogCommentExists(updateReqVO.getId());
        // 更新
        WorkLogCommentDO updateObj = BeanUtils.toBean(updateReqVO, WorkLogCommentDO.class);
        workLogCommentMapper.updateById(updateObj);
    }

    @Override
    public void deleteWorkLogComment(Long id) {
        // 校验存在
//        validateWorkLogCommentExists(id);
        // 删除
        workLogCommentMapper.deleteById(id);
    }

//    private void validateWorkLogCommentExists(Long id) {
//        if (workLogCommentMapper.selectById(id) == null) {
//            throw exception(WORK_LOG_COMMENT_NOT_EXISTS);
//        }
//    }

    @Override
    public WorkLogCommentDO getWorkLogComment(Long id) {
        return workLogCommentMapper.selectById(id);
    }

    @Override
    public PageResult<WorkLogCommentDO> getWorkLogCommentPage(WorkLogCommentPageReqVO pageReqVO) {
        return workLogCommentMapper.selectPage(pageReqVO);
    }

    @Override
    public IPage<CommentPageListVO> queryCommentPageList(Page<CommentPageListVO> page, CommentDTO dto) {
        IPage<CommentPageListVO> pageList = workLogCommentMapper.queryCommentPageList(page, dto);
        if (dto.getIsShowWorkLogContent() != null && dto.getIsShowWorkLogContent() == 1) {
            List<CommentPageListVO> records = pageList.getRecords();
            //模版ids过滤
            List<Long> workFormIds = records.stream().map(CommentPageListVO::getWorkFormId).collect(Collectors.toList());
            // TODO: 2024/4/11 查询模版列表

            pageList.getRecords().forEach(item -> {
                // TODO: 2024/4/11 - 这里需要通过模版id
                item.setWorkLogContent(null);
            });
        }
        return pageList;
    }


    public static void main(String[] args) {
        List<CommentPageListVO> records = Arrays.asList(
                new CommentPageListVO().setWorkLogId(1L).setWorkFormId("1"),
                new CommentPageListVO().setWorkLogId(1L).setWorkFormId("1"),
                new CommentPageListVO().setWorkLogId(1L).setWorkFormId("1"),
                new CommentPageListVO().setWorkLogId(2L).setWorkFormId("1"));
        Map<Long, String> collect = records.stream().collect(Collectors.toMap(CommentPageListVO::getWorkLogId, CommentPageListVO::getWorkFormId));
        System.out.println(collect);

    }

}