package cn.iocoder.yudao.module.system.service.fixed;

import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import cn.iocoder.yudao.module.system.controller.admin.fixed.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.fixed.AttendanceFixedDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import cn.iocoder.yudao.module.system.dal.mysql.fixed.AttendanceFixedMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 固定班制考勤设置 Service 实现类
 *
 * @author 艾楷
 */
@Service("fixed")
@Validated
public class AttendanceFixedServiceImpl implements AttendanceFixedService {

    @Resource
    private AttendanceFixedMapper fixedMapper;

    @Override
    public Long createFixed(AttendanceFixedSaveReqVO createReqVO) {
        // 插入
        AttendanceFixedDO fixed = BeanUtils.toBean(createReqVO, AttendanceFixedDO.class);
        fixedMapper.insert(fixed);
        // 返回
        return fixed.getId();
    }

    @Override
    public void updateFixed(AttendanceFixedSaveReqVO updateReqVO) {
        // 校验存在
        validateFixedExists(updateReqVO.getId());
        // 更新
        AttendanceFixedDO updateObj = BeanUtils.toBean(updateReqVO, AttendanceFixedDO.class);
        fixedMapper.updateById(updateObj);
    }

    @Override
    public void deleteFixed(Long id) {
        // 校验存在
        validateFixedExists(id);
        // 删除
        fixedMapper.deleteById(id);
    }

    private void validateFixedExists(Long id) {
        if (fixedMapper.selectById(id) == null) {
            throw exception(FIXED_NOT_EXISTS);
        }
    }

    @Override
    public AttendanceFixedDO getFixed(Long id) {
        return fixedMapper.selectById(id);
    }

    @Override
    public PageResult<AttendanceFixedDO> getFixedPage(AttendanceFixedPageReqVO pageReqVO) {
        return fixedMapper.selectPage(pageReqVO);
    }

}