package cn.iocoder.yudao.module.system.service.worklog;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstancePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceSaveReqVO;
import cn.iocoder.yudao.module.system.convert.worklog.LogInstanceConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.permission.RoleDO;
import cn.iocoder.yudao.module.system.dal.dataobject.permission.UserRoleDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogInstanceDO;
import cn.iocoder.yudao.module.system.dal.mysql.permission.UserRoleMapper;
import cn.iocoder.yudao.module.system.dal.mysql.user.AdminUserMapper;
import cn.iocoder.yudao.module.system.dal.mysql.worklog.LogInstanceMapper;
import cn.iocoder.yudao.module.system.enums.permission.DataScopeEnum;
import cn.iocoder.yudao.module.system.service.dept.DeptService;
import cn.iocoder.yudao.module.system.service.permission.MenuService;
import cn.iocoder.yudao.module.system.service.permission.PermissionService;
import cn.iocoder.yudao.module.system.service.permission.RoleService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LOG_INSTANCE_NOT_EXISTS;

/**
 * 日志实例的拓展 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class LogInstanceServiceImpl implements LogInstanceService {

    @Resource
    private LogInstanceMapper logInstanceMapper;

    @Resource
    private MenuService menuService;

    @Resource
    private PermissionService permissionService;

    @Resource
    private RoleService roleService;

    @Resource
    private UserRoleMapper userRoleMapper;

    @Resource
    private AdminUserMapper adminUserMapper;

    @Resource
    private DeptService deptService;

    @Override
    public Long createLogInstance(LogInstanceSaveReqVO createReqVO) {
        // 插入
        LogInstanceDO logInstance = BeanUtils.toBean(createReqVO, LogInstanceDO.class);
        logInstanceMapper.insert(logInstance);

        //根据查询权限获取menuID
        List<Long> menuIds = menuService.getMenuIdListByPermissionFromCache("system:view-log:query");
        if (menuIds != null && !menuIds.isEmpty()) {

            Map<String, String> readStatus = new HashMap<>();
            List<Long> roles = new ArrayList<>();

            //根据menuID 获取 roleIds
            Set<Long> roleIds = permissionService.getMenuRoleIdListByMenuIdFromCache(menuIds.get(0));

            List<RoleDO> roleDOS = roleService.getRoleList(roleIds);
            for (RoleDO roleDo : roleDOS) {

                if (Objects.equals(roleDo.getDataScope(), DataScopeEnum.ALL.getScope())) {                   //数据权限是全部时

                    roles.add(roleDo.getId());
                }else if (Objects.equals(roleDo.getDataScope(), DataScopeEnum.DEPT_CUSTOM.getScope())) {     //数据权限是指定部门

                    //指定部门组里存在 发起人的部门
                    if (roleDo.getDataScopeDeptIds().contains(createReqVO.getDeptId())) {

                        roles.add(roleDo.getId());
                    }
                }else if (Objects.equals(roleDo.getDataScope(), DataScopeEnum.DEPT_ONLY.getScope())) {       //数据权限是本部门

                    List<AdminUserDO> userDos = adminUserMapper.selectListByRoleId(roleDo.getId());
                    for (AdminUserDO userDO : userDos) {

                        //权限部门与发起人部门一致时
                        if (Objects.equals(userDO.getDeptId(), createReqVO.getDeptId())) {

                            readStatus.put(String.valueOf(userDO.getId()), "0");
                        }
                    }
                }else if (Objects.equals(roleDo.getDataScope(), DataScopeEnum.DEPT_AND_CHILD.getScope())) {    //数据权限是本部门及一下

                    //根据规则ID 查询用户信息
                    List<AdminUserDO> userDos = adminUserMapper.selectListByRoleId(roleDo.getId());
                    for (AdminUserDO userDO : userDos) {

                        Set<Long> deptIds = deptService.getChildDeptIdListFromCache(userDO.getDeptId());
                        //权限部门与发起人部门一致时
                        if (deptIds.contains(createReqVO.getDeptId())) {

                            readStatus.put(String.valueOf(userDO.getId()), "0");
                        }
                    }
                }
            }

            //通过满足条件的roleId 查询所有用户信息
            List<UserRoleDO> userRoleDOS = userRoleMapper.selectListByRoleIds(roles);
            for (UserRoleDO userRoleDO : userRoleDOS) {

                readStatus.put(String.valueOf(userRoleDO.getUserId()), "0");
            }

            LogInstanceDO updateObj = new LogInstanceDO();
//            updateObj.setReadStatus(JsonObject.);
//            logInstanceMapper.updateById();
        }

        // 返回
        return logInstance.getId();
    }

    @Override
    public void updateLogInstance(LogInstanceSaveReqVO updateReqVO) {
        // 校验存在
        validateLogInstanceExists(updateReqVO.getId());
        // 更新
        LogInstanceDO updateObj = BeanUtils.toBean(updateReqVO, LogInstanceDO.class);
        logInstanceMapper.updateById(updateObj);
    }

    @Override
    public void deleteLogInstance(Long id) {
        // 校验存在
        validateLogInstanceExists(id);
        // 删除
        logInstanceMapper.deleteById(id);
    }

    private void validateLogInstanceExists(Long id) {
        if (logInstanceMapper.selectById(id) == null) {
            throw exception(LOG_INSTANCE_NOT_EXISTS);
        }
    }

    @Override
    public LogInstanceDO getLogInstance(Long id) {
        return logInstanceMapper.selectById(id);
    }

    @Override
    public PageResult<LogInstanceDO> getLogInstancePage(LogInstancePageReqVO pageReqVO) {
        return logInstanceMapper.selectPage(pageReqVO);
    }

    @Override
    public PageResult<LogInstanceRespVO> getMyProcessInstancePage(Long userId, LogInstancePageReqVO pageReqVO) {

        return LogInstanceConvert.INSTANCE.convertPage(logInstanceMapper.selectMyPage(userId, pageReqVO));
    }
}