package cn.iocoder.yudao.module.system.service.worklog;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.rule.LogRulePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.rule.LogRuleSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogRuleDO;
import cn.iocoder.yudao.module.system.dal.mysql.worklog.LogRuleMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LOG_RULE_NOT_EXISTS;

/**
 * 日志规则 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class LogRuleServiceImpl implements LogRuleService {

    @Resource
    private LogRuleMapper logRuleMapper;

    @Override
    public Long createLogRule(LogRuleSaveReqVO createReqVO) {
        // 插入
        LogRuleDO logRule = BeanUtils.toBean(createReqVO, LogRuleDO.class);
        logRuleMapper.insert(logRule);
        // 返回
        return logRule.getId();
    }

    @Override
    public void updateLogRule(LogRuleSaveReqVO updateReqVO) {
        // 校验存在
        validateLogRuleExists(updateReqVO.getId());
        // 更新
        LogRuleDO updateObj = BeanUtils.toBean(updateReqVO, LogRuleDO.class);
        logRuleMapper.updateById(updateObj);
    }

    @Override
    public void deleteLogRule(Long id) {
        // 校验存在
        validateLogRuleExists(id);
        // 删除
        logRuleMapper.deleteById(id);
    }

    private void validateLogRuleExists(Long id) {
        if (logRuleMapper.selectById(id) == null) {
            throw exception(LOG_RULE_NOT_EXISTS);
        }
    }

    @Override
    public LogRuleDO getLogRule(Long id) {
        return logRuleMapper.selectById(id);
    }

    @Override
    public PageResult<LogRuleDO> getLogRulePage(LogRulePageReqVO pageReqVO) {
        return logRuleMapper.selectPage(pageReqVO);
    }

}