package cn.iocoder.yudao.module.smartfactory.service.factorydata;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.FactoryDataPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorydata.vo.FactoryDataSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factorydata.FactoryDataDO;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;

/**
 * 工厂出库入库数据 Service 接口
 *
 * @author 艾楷
 */
public interface FactoryDataService {

    /**
     * 创建工厂出库入库数据
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createData(@Valid FactoryDataSaveReqVO createReqVO);

    /**
     * 更新工厂出库入库数据
     *
     * @param updateReqVO 更新信息
     */
    void updateData(@Valid FactoryDataSaveReqVO updateReqVO);

    /**
     * 删除工厂出库入库数据
     *
     * @param id 编号
     */
    void deleteData(Long id);

    /**
     * 获得工厂出库入库数据
     *
     * @param id 编号
     * @return 工厂出库入库数据
     */
    FactoryDataDO getData(Long id);

    /**
     * 获得工厂出库入库数据分页
     *
     * @param pageReqVO 分页查询
     * @return 工厂出库入库数据分页
     */
    PageResult<FactoryDataDO> getDataPage(FactoryDataPageReqVO pageReqVO);

    /**
     * 获取当天的所有数据
     *
     * @return
     */
    List<FactoryDataDO> getTodayInboundAndOutboundOverview(Date time);

    /**
     * 根据日期和工厂过滤
     *
     * @param factoryId
     * @param beginDay
     * @param endDay
     * @return
     */
    List<FactoryDataDO> getByTimeAndFactoryId(Long factoryId, Date beginDay, Date endDay);
}