package cn.iocoder.yudao.module.smartfactory.service.packagedata;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packagedata.vo.PackageDataPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.packagedata.vo.PackageDataSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.packagedata.PackageDataDO;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;

/**
 * 打包线数据 Service 接口
 *
 * @author 艾楷
 */
public interface PackageDataService {

    /**
     * 创建打包线数据
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPackageData(@Valid PackageDataSaveReqVO createReqVO);

    /**
     * 更新打包线数据
     *
     * @param updateReqVO 更新信息
     */
    void updatePackageData(@Valid PackageDataSaveReqVO updateReqVO);

    /**
     * 删除打包线数据
     *
     * @param id 编号
     */
    void deletePackageData(Long id);

    /**
     * 获得打包线数据
     *
     * @param id 编号
     * @return 打包线数据
     */
    PackageDataDO getPackageData(Long id);

    /**
     * 获得打包线数据分页
     *
     * @param pageReqVO 分页查询
     * @return 打包线数据分页
     */
    PageResult<PackageDataDO> getPackageDataPage(PackageDataPageReqVO pageReqVO);

    /**
     * 列表
     *
     * @param factoryId
     * @return
     */
    List<PackageDataDO> getFactoryPackLineInfo(Long factoryId);

    /**
     * 列表 - 时间参数
     *
     * @param factoryId
     * @param beginTime
     * @param endTime
     * @return
     */
    List<PackageDataDO> getFactoryPackLineInfoByTime(Long factoryId, Date beginTime, Date endTime);
}