package cn.iocoder.yudao.module.smartfactory.service.staff;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staff.vo.StaffPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staff.vo.StaffSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.screendata.factory.vo.StaffDataRespVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.staff.StaffDO;

import javax.validation.Valid;

/**
 * 员工 Service 接口
 *
 * @author 艾楷
 */
public interface StaffService {

    /**
     * 创建员工
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createStaff(@Valid StaffSaveReqVO createReqVO);

    /**
     * 更新员工
     *
     * @param updateReqVO 更新信息
     */
    void updateStaff(@Valid StaffSaveReqVO updateReqVO);

    /**
     * 删除员工
     *
     * @param id 编号
     */
    void deleteStaff(Long id);

    /**
     * 获得员工
     *
     * @param id 编号
     * @return 员工
     */
    StaffDO getStaff(Long id);

    /**
     * 获得员工分页
     *
     * @param pageReqVO 分页查询
     * @return 员工分页
     */
    PageResult<StaffDO> getStaffPage(StaffPageReqVO pageReqVO);

    /**
     * 查询工厂员工信息
     *
     * @param factoryId
     * @return
     */
    StaffDataRespVO getStaffData(Long factoryId);

    /**
     * 获取人员数量
     * @return
     */
    Integer getCount();
}